@@ -45,10 +45,38 @@ DEFAULT_AIRR_FIELDS = (
     "consensus_count",
     "duplicate_count",
 )
+DEFAULT_10X_FIELDS = DEFAULT_AIRR_FIELDS + ("is_cell", "high_confidence")
 DEFAULT_AIRR_CELL_ATTRIBUTES = ("is_cell", "high_confidence", "multi_chain")
 
 
-def _read_10x_vdj_json(path: Union[str, Path], filtered: bool = True) -> AnnData:
+def _cdr3_from_junction(junction_aa, junction_nt):
+    """CDR3 euqals junction without the conserved residues C and W/F, respectively.
+    Should the conserved residues not equal to C and W/F, then the chain
+    is non-productive and we set CDR3 to None.
+
+    See also https://github.com/icbi-lab/scirpy/pull/290.
+    """
+    cdr3_aa, cdr3_nt = None, None
+    if (
+        junction_aa is not None
+        and junction_aa[0] == "C"
+        and junction_aa[-1] in ("W", "F")
+    ):
+        cdr3_aa = junction_aa[1:-1]
+    if (
+        junction_nt is not None
+        and _translate_dna_to_protein(junction_nt[:3]) == "C"
+        and _translate_dna_to_protein(junction_nt[-3:]) in ("W", "F")
+    ):
+        cdr3_nt = junction_nt[3:-3]
+    return cdr3_aa, cdr3_nt
+
+
+def _read_10x_vdj_json(
+    path: Union[str, Path],
+    filtered: bool = True,
+    include_fields: Optional[Collection[str]] = None,
+) -> AnnData:
     """Read IR data from a 10x genomics `all_contig_annotations.json` file"""
     logger = _IOLogger()
     with open(path, "r") as f:
@@ -137,12 +165,29 @@ def _read_10x_vdj_json(path: Union[str, Path], filtered: bool = True) -> AnnData
         chain["is_cell"] = cell["is_cell"]
         chain["high_confidence"] = cell["high_confidence"]
 
+        # additional cols from CR6 outputs: fwr{1,2,3,4}{,_nt} and cdr{1,2}{,_nt}
+        fwrs = [f"fwr{i}" for i in range(1, 5)]
+        cdrs = [f"cdr{i}" for i in range(1, 3)]
+
+        for col in fwrs + cdrs:
+            if col in cell.keys():
+                chain[col] = cell[col].get("nt_seq") if cell[col] else None
+                chain[col + "_aa"] = cell[col].get("aa_seq") if cell[col] else None
+
+        chain["cdr3_aa"], chain["cdr3"] = _cdr3_from_junction(
+            chain["junction_aa"], chain["junction"]
+        )
+
         ir_obj.add_chain(chain)
 
-    return from_airr_cells(airr_cells.values())
+    return from_airr_cells(airr_cells.values(), include_fields=include_fields)
 
 
-def _read_10x_vdj_csv(path: Union[str, Path], filtered: bool = True) -> AnnData:
+def _read_10x_vdj_csv(
+    path: Union[str, Path],
+    filtered: bool = True,
+    include_fields: Optional[Collection[str]] = None,
+) -> AnnData:
     """Read IR data from a 10x genomics `_contig_annotations.csv` file"""
     logger = _IOLogger()
     df = pd.read_csv(path)
@@ -170,15 +215,34 @@ def _read_10x_vdj_csv(path: Union[str, Path], filtered: bool = True) -> AnnData:
                 is_cell=chain_series["is_cell"],
                 high_confidence=chain_series["high_confidence"],
             )
+
+            # additional cols from CR6 outputs: fwr{1,2,3,4}{,_nt} and cdr{1,2}{,_nt}
+            fwrs = [f"fwr{i}" for i in range(1, 5)]
+            cdrs = [f"cdr{i}" for i in range(1, 3)]
+
+            for col in fwrs + cdrs:
+                if col in chain_series.index:
+                    chain_dict[col + "_aa"] = chain_series.get(col)
+                if col + "_nt" in chain_series.index:
+                    chain_dict[col] = chain_series.get(col + "_nt")
+
+            chain_dict["cdr3_aa"], chain_dict["cdr3"] = _cdr3_from_junction(
+                chain_dict["junction_aa"], chain_dict["junction"]
+            )
+
             ir_obj.add_chain(chain_dict)
 
         airr_cells[barcode] = ir_obj
 
-    return from_airr_cells(airr_cells.values())
+    return from_airr_cells(airr_cells.values(), include_fields=include_fields)
 
 
-@_doc_params(doc_working_model=doc_working_model)
-def read_10x_vdj(path: Union[str, Path], filtered: bool = True) -> AnnData:
+@_doc_params(doc_working_model=doc_working_model, include_fields=DEFAULT_10X_FIELDS)
+def read_10x_vdj(
+    path: Union[str, Path],
+    filtered: bool = True,
+    include_fields: Optional[Collection[str]] = DEFAULT_10X_FIELDS,
+) -> AnnData:
     """\
     Read :term:`IR` data from 10x Genomics cell-ranger output.
 
@@ -200,6 +264,13 @@ def read_10x_vdj(path: Union[str, Path], filtered: bool = True) -> AnnData:
         Only keep filtered contig annotations (i.e. `is_cell` and `high_confidence`).
         If using `filtered_contig_annotations.csv` already, this option
         is futile.
+    include_fields
+        The fields to include in `adata`. The AIRR rearrangment schema contains
+        can contain a lot of columns, most of which irrelevant for most analyses.
+        Per default, this includes a subset of columns relevant for a typical
+        scirpy analysis, to keep `adata.obs` a bit cleaner. Defaults to {include_fields}.
+        Set this to `None` to include all columns.
+    
 
     Returns
     -------
@@ -208,9 +279,9 @@ def read_10x_vdj(path: Union[str, Path], filtered: bool = True) -> AnnData:
     """
     path = Path(path)
     if path.suffix == ".json":
-        return _read_10x_vdj_json(path, filtered)
+        return _read_10x_vdj_json(path, filtered, include_fields)
     else:
-        return _read_10x_vdj_csv(path, filtered)
+        return _read_10x_vdj_csv(path, filtered, include_fields)
 
 
 @_doc_params(doc_working_model=doc_working_model)
@@ -342,7 +413,7 @@ def read_airr(
     """\
     Read data from `AIRR rearrangement <https://docs.airr-community.org/en/latest/datarep/rearrangements.html>`_ format.
 
-    The following columns are required by scirpy: 
+    The following columns are required by scirpy:
      * `cell_id`
      * `productive`
      * `locus`
@@ -351,8 +422,8 @@ def read_airr(
 
     Data should still import if one of these fields is missing, but they are required
     by most of scirpy's processing functions. All chains for which the field
-    `junction_aa` is missing or empty, will be considered as non-productive and 
-    will be moved to the `extra_chains` column.   
+    `junction_aa` is missing or empty, will be considered as non-productive and
+    will be moved to the `extra_chains` column.
 
     {doc_working_model}
 
@@ -370,15 +441,15 @@ def read_airr(
     infer_locus
         Try to infer the `locus` column from gene names, in case it is not specified.
     cell_attributes
-        Fields in the rearrangement schema that are specific for a cell rather 
-        than a chain. The values must be identical over all records belonging to a 
-        cell. This defaults to {cell_attributes}. 
+        Fields in the rearrangement schema that are specific for a cell rather
+        than a chain. The values must be identical over all records belonging to a
+        cell. This defaults to {cell_attributes}.
     include_fields
         The fields to include in `adata`. The AIRR rearrangment schema contains
-        can contain a lot of columns, most of which irrelevant for most analyses. 
-        Per default, this includes a subset of columns relevant for a typical 
-        scirpy analysis, to keep `adata.obs` a bit cleaner. Defaults to {include_fields}. 
-        Set this to `None` to include all columns. 
+        can contain a lot of columns, most of which irrelevant for most analyses.
+        Per default, this includes a subset of columns relevant for a typical
+        scirpy analysis, to keep `adata.obs` a bit cleaner. Defaults to {include_fields}.
+        Set this to `None` to include all columns.
 
     Returns
     -------
@@ -674,7 +745,7 @@ def from_dandelion(dandelion, transfer: bool = False, **kwargs) -> AnnData:
     """\
     Import data from `Dandelion <https://github.com/zktuong/dandelion>`_ (:cite:`Stephenson2021`).
 
-    Internally calls :func:`scirpy.io.read_airr`. 
+    Internally calls :func:`scirpy.io.read_airr`.
 
     {doc_working_model}
 
@@ -686,7 +757,7 @@ def from_dandelion(dandelion, transfer: bool = False, **kwargs) -> AnnData:
         Whether to execute `dandelion.tl.transfer` to transfer all data
         to the :class:`anndata.AnnData` instance.
     **kwargs
-        Additional arguments passed to :func:`scirpy.io.read_airr`. 
+        Additional arguments passed to :func:`scirpy.io.read_airr`.
 
     Returns
     -------
